/*
 * Copyright (C) 1997-2000 SpellCaster Telecommunications Inc.
 * $Id: call.h,v 1.3 2004/08/15 23:22:03 bcrl Exp $
 * Released under the GNU Public License. See LICENSE file for details.
 */


#ifndef CALL_H
#define CALL_H

#include "d_aps_if.h"

#include "ctrlfd.h"

extern unsigned int callIdent;

class CInterface;
class OptionSet_t;

class Call : public ctrlfd_die_listener {
public:
	CInterface	*m_iface;
	int		m_wait_for_hangup;
	unsigned int	ident;	/* semi-unique 'handle' for user apps to refer to the call */

	ctrlfd_t	*cfd;
	channel_t	*ch;
	OptionSet_t	*options;

	virtual void ctrlfd_die_notification(ctrlfd_t *cfd) {
		cfd = NULL;
	}

#define MAX_CALL_USERS	16
	Call **call_users[MAX_CALL_USERS];
	int n_call_users;

	void CallUser(Call **callp) {
		if (n_call_users >= MAX_CALL_USERS)
			*(char *)0 = 0;
		call_users[n_call_users++] = callp;
		fprintf(stderr, "CallUser(%p[%d])\n", callp, n_call_users);
	}

	void DiscardCallUser(Call **callp) {
		int i;
		if (n_call_users <= 0)
			*(char *)0 = 0;

		for (i=0; i<n_call_users; i++) {
			if (call_users[i] == callp) {
				call_users[i] = call_users[--n_call_users];
				call_users[n_call_users] = NULL;
				*callp = NULL;
				return;
			}
		}

		fprintf(stderr, "DiscardCallUser(call=%p, callp=%p) - huh?\n",
			this, callp);
	}

	Call(int wait_for_hangup = 0, CInterface *iface = NULL) {
		m_iface = iface;
		m_wait_for_hangup = wait_for_hangup;
		ident = ++callIdent;
		cfd = NULL;
		ch = NULL;
		options = NULL;

		n_call_users = 0;
		memset(call_users, 0, sizeof(call_users));
	}

	void DiscardOptions(void);

	virtual ~Call() {
		int i;
		for (i=0; i<n_call_users; i++)
			*call_users[i] = NULL;
		DiscardOptions();
		return;
	}
};

class DialResponse_t {
public:
	Call *call;
	int status;
	char msg[256];

	DialResponse_t()
	{
		status = 0;
		msg[0] = 0;
		call = 0;
	}

	DialResponse_t(Call *c)
	{
		status = 0;
		msg[0] = 0;
		call = c;
	}
};

#endif /* ndef CALL_H */

